from typing import Any
from helper_i18n.utils import load_i18n_file


class I18nHandler:
    def __init__(self, i18npaths: dict[str, str], default_language: str) -> None:
        self.i18n = load_i18n_file(i18npaths)
        self.__define_default_language(default_language)

    def get_exception_message(
        self, key: str, language: str, params: dict[str, Any]
    ) -> str:
        language = self.__get_language(language)
        exception = self.i18n[language]["exception"].get(key, key)
        return exception.format(**params)

    def get_message(self, key: str, language: str, params: dict[str, Any]) -> str:
        language = self.__get_language(language)
        message = self.i18n[language]["message"].get(key, key)
        return message.format(**params)

    def __define_default_language(self, language: str) -> None:
        if language not in self.i18n:
            raise ValueError(f"Language {language} not found in i18n files")
        self.default_language = language

    def __get_language(self, language: str | None = None) -> str:
        languages = language.split(",") if language else []
        for language in languages:
            if language in self.i18n:
                return language
        return self.default_language



import json
from typing import Any

from helper_i18n.typing import I18nDict


def check_i18n_file_format(i18n_json: dict[str, Any]) -> bool:
    if "exception" not in i18n_json or "message" not in i18n_json:
        return False
    if not isinstance(i18n_json["exception"], dict) or not isinstance(
        i18n_json["message"], dict
    ):
        return False
    return True


def load_i18n_file(i18n_paths: dict[str, str]) -> dict[str, I18nDict]:
    i18_n_json: dict[str, I18nDict] = {}
    for language, i18n_path in i18n_paths.items():
        with open(i18n_path, "r") as file:
            i18n_json = json.load(file)
        if not check_i18n_file_format(i18n_json):
            raise ValueError(f"Invalid i18n file format {i18n_path}")
        i18_n_json[language] = i18n_json
    return i18_n_json



from http import HTTPStatus
from typing import Any, Optional

from helper_i18n.typing import errors


class CustomException(Exception):
    def __init__(
        self,
        status_code: int,
        i8n_key: str,
        errors: Optional[errors] = None,
        **params: Any,
    ) -> None:
        self.status_code = status_code
        self.i8n_key = i8n_key
        self.errors = errors
        self.params = params
        super().__init__()

    def to_dict(self) -> dict[str, Any]:
        error = HTTPStatus(self.status_code).phrase
        return {
            "example": {
                "error": error,
                "message": self.i8n_key,
                "path": "/path",
                "errors": [],
            }
        }



from typing import Any, Callable
from helper_i18n.utils import load_i18n_file, check_i18n_file_format
import pytest
from pytest_mock import MockFixture
from json import dumps

mock_i18n: dict[str, Any] = {"exception": {}, "message": {}}


@pytest.fixture
def load_sut(mocker: MockFixture):
    mocker.patch("builtins.open", mocker.mock_open(read_data=dumps(mock_i18n)))
    mocker.patch("json.loads", return_value=mock_i18n)
    return load_i18n_file


@pytest.fixture
def check_sut():
    return check_i18n_file_format

def test_load_i18n_file_should_try_open_18n_file(
    load_sut: Callable[[Any], Any], mocker: MockFixture
):
    open_spy = mocker.patch(
        "builtins.open", mocker.mock_open(read_data=dumps(mock_i18n))
    )

    load_sut("test")

    open_spy.assert_called_once_with("test", "r")


def test_handler_should_load_i18n_file(load_sut: Callable[[Any], Any], mocker: MockFixture):
    load_spy = mocker.patch("json.loads", return_value=mock_i18n)

    load_sut("test")

    load_spy.assert_called_once_with(dumps(mock_i18n))


def test_should_raise_exception_when_invalid_i18n_file_format(
    load_sut: Callable[[Any], Any], mocker: MockFixture
):
    mocker.patch("helper_i18n.utils.check_i18n_file_format", return_value=False)

    with pytest.raises(ValueError):
        load_sut("test")

def test_should_return_i18n_file_when_valid_i18n_file_format(
    load_sut: Callable[[Any], Any], mocker: MockFixture
):
    mocker.patch("helper_i18n.utils.check_i18n_file_format", return_value=True)

    assert load_sut("test") == mock_i18n

def test_should_return_false_when_i18n_file_doent_have_exception_or_message(
    check_sut: Callable[[dict[str, Any]], bool]
):
    assert check_sut({'exception':{}}) is False
    assert check_sut({'message':{}}) is False

def test_should_return_false_when_i18n_file_exception_or_message_is_not_dict(
    check_sut: Callable[[dict[str, Any]], bool]
):
    assert check_sut({'exception':[], 'message':{}}) is False
    assert check_sut({'exception':{}, 'message':[]}) is False

def test_should_return_true_when_i18n_file_is_valid(check_sut: Callable[[dict[str, Any]], bool]):
    assert check_sut(mock_i18n) is True


from typing import Any, Optional, cast
from starlette.applications import Starlette
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from starlette.requests import Request
from http import HTTPStatus

from helper_i18n.typing import errors
from helper_i18n import CustomException
from helper_i18n.handler.i18n_handler import I18nHandler


class ExceptionHandler:
    def __init__(self, i18n_handler: I18nHandler) -> None:
        self.i18n = i18n_handler

    def include_app(self, app: Starlette):
        app.add_exception_handler(
            RequestValidationError, self.__include_pydantic_exception
        )
        app.add_exception_handler(CustomException, self.__include_custom_exception)

    def __include_pydantic_exception(
        self, request: Request, exception: Exception
    ) -> JSONResponse:
        exception = cast(RequestValidationError, exception)
        errors: list[dict[str, Any]] = []
        for error in exception.errors():
            error_name = error["loc"][1]
            if isinstance(error_name, int):
                error_name = error["loc"][0]
            errors.append(
                {
                    error_name: error["type"],
                }
            )

        return self.__create_response(
            exception.__class__.__name__, request, 422, errors
        )

    def __include_custom_exception(
        self, request: Request, exception: Exception
    ) -> JSONResponse:
        exception = cast(CustomException, exception)
        return self.__create_response(
            exception.i8n_key,
            request,
            exception.status_code,
            exception.errors,
            **exception.params,
        )

    def __create_response(
        self,
        i18n_key: str,
        request: Request,
        status_code: int,
        errors: Optional[errors] = None,
        **params: Any,
    ) -> JSONResponse:
        error = HTTPStatus(status_code).phrase
        language = self.__get_language(request)
        i18n_message = self.i18n.get_exception_message(i18n_key, language, params)
        content: dict[str, list[dict[str, str]] | str] = {
            "error": error,
            "message": i18n_message,
            "path": request.url.path,
        }
        if errors:
            content["errors"] = errors
        return JSONResponse(content=content, status_code=status_code)

    def __get_language(self, request: Request) -> str:
        return request.headers.get("accept-language", self.i18n.default_language)



from typing import Type
import pytest
from pytest_mock import MockFixture
from helper_i18n.handler.i18n_handler import I18nHandler


@pytest.fixture
def factory_sut():
    return I18nHandler

@pytest.fixture
def sut(mocker: MockFixture):
    mocker.patch("helper_i18n.handler.i18n_handler.load_i18n_file", return_value={"exception": {"valid_key":"valid_value"}, "message": {"valid_key":"valid_value"}})
    return I18nHandler("test")
    

def test_should_use_load_i18n_file_with_i18n_path(factory_sut:Type[I18nHandler], mocker: MockFixture):
    load_spy = mocker.patch("helper_i18n.handler.i18n_handler.load_i18n_file")

    factory_sut("test")

    load_spy.assert_called_once_with("test")
    
def test_should_throw_exception_when_i18n_file_format_is_invalid(factory_sut:Type[I18nHandler], mocker: MockFixture):
    mocker.patch("helper_i18n.handler.i18n_handler.load_i18n_file", side_effect=ValueError)

    with pytest.raises(ValueError):
        factory_sut("test")

def test_should_return_message_in_get_exception_message_if_exist(sut: I18nHandler):
    assert sut.get_exception_message("valid_key") == "valid_value"
    
def test_should_return_key_in_get_exception_message_if_not_exist(sut: I18nHandler):
    assert sut.get_exception_message("invalid_key") == "invalid_key"
    
def test_should_return_message_in_get_message_if_exist(sut: I18nHandler):
    assert sut.get_message("valid_key") == "valid_value"
    
def test_should_return_key_in_get_message_if_not_exist(sut: I18nHandler):
    assert sut.get_message("invalid_key") == "invalid_key"


from typing import Any, Mapping, Sequence, Type

from pydantic import BaseModel
from helper_i18n import CustomException


class DocsHandler:
    def __init__(
        self,
        exceptions: Sequence[CustomException],
        responses: Sequence[Mapping[int, Type[BaseModel]]],
    ) -> None:
        self.exceptions = exceptions
        self.responses = responses

    def get_docs(self) -> dict[Any, dict[str, Any]] | None:
        docs: dict[Any, dict[str, Any]] = {}
        exceptions = self.get_exceptions()
        responses = self.get_responses()
        docs.update(exceptions)
        docs.update(responses)
        return docs

    def get_exceptions(self) -> dict[int, dict[str, Any]]:
        exceptions: dict[int, dict[str, Any]] = {}
        for exception in self.exceptions:
            exceptions[exception.status_code] = self.get_exception_content(
                exception.to_dict()
            )
        return exceptions

    def get_responses(self) -> dict[int, dict[str, Any]]:
        responses: dict[int, dict[str, Any]] = {}
        for response in self.responses:
            for status_code, response_data in response.items():
                responses[status_code] = self.get_response_content(response_data)
        return responses

    def get_exception_content(self, exception_data: dict[str, Any]) -> dict[str, Any]:
        content: dict[str, Any] = {
            "content": {
                "application/json": exception_data,
            },
            "description": exception_data["example"]["message"],
        }
        return content

    def get_response_content(self, response_data: Type[BaseModel]) -> dict[str, Any]:
        content: dict[str, Any] = {"model": response_data}
        return content
